package de.ozgcloud.admin.settings;

import static org.assertj.core.api.Assertions.*;

import java.util.ArrayList;
import java.util.List;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.junit.jupiter.MockitoExtension;

@ExtendWith(MockitoExtension.class)
public class AdminEnvironmentRepositoryTest {

    @InjectMocks
    private AdminEnvironmentRepository repository;

    private ConfigurationProperties configurationProperties = ConfigurationPropertiesTestFactory.create();

    @Nested
    class TestBuildEnvironment {
        @Test
        void shouldHaveCorrectAppName() {

            var environment = repository.buildEnvironment(ConfigurationPropertiesTestFactory.APPNAME, new ArrayList<ConfigurationProperties>());

            assertThat(environment.getName()).isEqualTo(ConfigurationPropertiesTestFactory.APPNAME);
        }

        @Test
        void shouldHaveCorrectNumberOfPropertySources() {

            var environment = repository.buildEnvironment(ConfigurationPropertiesTestFactory.APPNAME,
                    List.of(configurationProperties, configurationProperties));

            assertThat(environment.getPropertySources()).hasSize(2);
        }

        @Test
        void shouldHandleEmptyConfigurationList() {

            var environment = repository.buildEnvironment(ConfigurationPropertiesTestFactory.APPNAME, new ArrayList<ConfigurationProperties>());

            assertThat(environment.getPropertySources()).isEmpty();

        }
    }

    @Nested
    class TestCreatePropertySource {
        @Test
        void shouldHaveCorrectSource() {

            var propertySource = repository.createPropertySource(configurationProperties);

            assertThat(propertySource.getSource()).isEqualTo(configurationProperties.getSource());
        }

        @Test
        void shouldHaveCorrectName() {

            var propertySource = repository.createPropertySource(configurationProperties);

            assertThat(propertySource.getName()).isNull();
        }
    }

}
