/*
 * Copyright (c) 2024. Das Land Schleswig-Holstein vertreten durch das Ministerium für Energiewende, Klimaschutz, Umwelt und Natur
 * Zentrales IT-Management
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.admin.settings;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.data.rest.RepositoryRestProperties;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.http.HttpStatus;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;

import de.ozgcloud.common.test.DataITCase;
import lombok.SneakyThrows;

@DataITCase
@AutoConfigureMockMvc
@WithMockUser
class SettingsITCase {

	@Autowired
	private MockMvc mockMvc;

	@Autowired
	private MongoOperations mongoOperations;

	@Autowired
	private RepositoryRestProperties restProperties;

	@Nested
	class TestRestRepo {

		@BeforeEach
		void init() {
			mongoOperations.dropCollection(Settings.class);
			mongoOperations.save(SettingsTestFactory.create());
		}

		@Test
		@SneakyThrows
		void shouldHaveStatusOkForExisting() {
			var id = mongoOperations.findAll(Settings.class).get(0).getId();

			var result = doPerform(id);

			result.andExpect(status().isOk());
		}

		@Test
		@SneakyThrows
		void shouldHaveStatusNotFoundForNonExisting() {
			var result = doPerform("unknown");

			result.andExpect(status().is(HttpStatus.NOT_FOUND.value()));
		}

		@SneakyThrows
		private ResultActions doPerform(String id) {
			return mockMvc.perform(get(String.join("/", restProperties.getBasePath(), SettingsConstants.PATH, id)));
		}
	}

}